/*
GNU General Public License

DSTC (Death Star Trash Compactor) Shell - Java ZIP Utility Frontend
Copyright (C) 2000  Howard A Kistler

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

package com.hexidec.tool;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.Calendar;
import java.util.Date;
import java.util.Enumeration;
import java.util.Vector;
import java.util.zip.ZipEntry;
import java.util.zip.ZipFile;
import java.util.zip.ZipOutputStream;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.table.AbstractTableModel;

import com.hexidec.tool.DSTC;

/** DSTC (Death Star Trash Compactor) Shell
  * Wrapper for DSTC ZIP manipulator
  *
  * @author Howard Kistler
  * @version 0.1
  *
  * requires Java 2 (JDK 1.3) & Swing
  *
  * VERSION HISTORY
  * 0.1 - initially creation (08/21/2000)
  */

public class DSTCshell extends JFrame implements WindowListener, ActionListener
{

	private DSTC dstc;

	private JMenuBar jMenuBar;
	private JMenu jMenuFile;
	private JMenu jMenuActn;
	private JTable jtblArchiveInfo;
	private ZipInfoTableModel zipTableModel;
	private JScrollPane jspViewport;

	private Calendar calen; // for decoding ZipEntry dates

	private File currentArchive;

	private final String progName  = "DSTC";
	private final String[] extsZip = { "zip", "gzip", "gz" };

//  CONSTRUCTORS ----------------------------------------------------------------->

	public DSTCshell()
	{
		super();

		dstc = new DSTC();

		calen = Calendar.getInstance();

		currentArchive = new File(".");

		jMenuFile = new JMenu("File");
		JMenuItem jmiNew  = new JMenuItem("New");  jmiNew.setActionCommand("new");   jmiNew.addActionListener(this);  jMenuFile.add(jmiNew);
		JMenuItem jmiOpen = new JMenuItem("Open"); jmiOpen.setActionCommand("open"); jmiOpen.addActionListener(this); jMenuFile.add(jmiOpen);
		JMenuItem jmiSave = new JMenuItem("Save"); jmiSave.setActionCommand("save"); jmiSave.addActionListener(this); jMenuFile.add(jmiSave);
		JMenuItem jmiExit = new JMenuItem("Exit"); jmiExit.setActionCommand("exit"); jmiExit.addActionListener(this); jMenuFile.add(jmiExit);

		jMenuActn = new JMenu("Action");
		JMenuItem jmiAddFile  = new JMenuItem("Add File");     jmiAddFile.setActionCommand("addtoarchive"); jmiAddFile.addActionListener(this);  jMenuActn.add(jmiAddFile);
		JMenuItem jmiExtractF = new JMenuItem("Extract File"); jmiExtractF.setActionCommand("extractfile"); jmiExtractF.addActionListener(this); jMenuActn.add(jmiExtractF);
		JMenuItem jmiExtractA = new JMenuItem("Extract All");  jmiExtractA.setActionCommand("extractall");  jmiExtractA.addActionListener(this); jMenuActn.add(jmiExtractA);

		jMenuBar = new JMenuBar();
		jMenuBar.add(jMenuFile);
		jMenuBar.add(jMenuActn);
		this.setJMenuBar(jMenuBar);

		zipTableModel = new ZipInfoTableModel();
		jtblArchiveInfo = new JTable(zipTableModel);
		jtblArchiveInfo.setShowVerticalLines(true);
		jtblArchiveInfo.setShowHorizontalLines(false);
		jspViewport = new JScrollPane(jtblArchiveInfo);
		jspViewport.setBackground(jtblArchiveInfo.getBackground());

		this.getContentPane().add(jspViewport);

		this.addWindowListener(this);
		this.pack();
		this.show();
		setAppTitle("");
	}

//  LISTENER METHODS ------------------------------------------------------------->

	/* WindowListener methods */
	public void windowClosing(WindowEvent we)
	{
		this.dispose();
		System.exit(0);
	}
	public void windowOpened(WindowEvent we)      { ; }
	public void windowClosed(WindowEvent we)      { ; }
	public void windowActivated(WindowEvent we)   { ; }
	public void windowDeactivated(WindowEvent we) { ; }
	public void windowIconified(WindowEvent we)   { ; }
	public void windowDeiconified(WindowEvent we) { ; }

	/* ActionListener method */
	public void actionPerformed(ActionEvent ae)
	{
		String command = ae.getActionCommand();
		try
		{
			if(command.equals("new"))
			{
				sysMsg("-> NEW");
			}
			else if(command.equals("open"))
			{
				scanArchive();
			}
			else if(command.equals("save"))
			{
				saveArchive();
			}
			else if(command.equals("exit"))
			{
				this.dispose();
				System.exit(0);
			}
			else if(command.equals("addtoarchive"))
			{
				addToArchive();
			}
			else if(command.equals("extractfile"))
			{
				extractFileFromArchive();
			}
			else if(command.equals("extractall"))
			{
				extractAllFromArchive();
			}
		}
		catch(FileNotFoundException fnfe)
		{
			sysMsg("FileNotFoundException : " + fnfe.getMessage());
		}
		catch(IOException ioe)
		{
			sysMsg("IOException : " + ioe.getMessage());
		}

		this.repaint();
	}

//  CLASS METHODS ---------------------------------------------------------------->

	private void scanArchive() throws FileNotFoundException, IOException
	{
		JFileChooser loadDialog = new JFileChooser(".");
		loadDialog.setDialogType(JFileChooser.OPEN_DIALOG);
		loadDialog.setFileFilter(new MutableFilter(extsZip, "Zip Archives (*.zip, *.gzip, *.gz)"));
		int optionSelected = loadDialog.showOpenDialog(this);
		if(optionSelected == JFileChooser.APPROVE_OPTION)
		{
			currentArchive = loadDialog.getSelectedFile();
			openArchive(currentArchive);
			setAppTitle(currentArchive.getName());
		}
	}

	private void openArchive(File zipArchive) throws FileNotFoundException, IOException
	{
		Vector vcScan = dstc.scanZip(zipArchive);
		Object[][] newTableData = new Object[vcScan.size()][zipTableModel.getColumnCount()];
		for(int i = 0; i < vcScan.size(); i++)
		{
			ZipEntry zEntry = (ZipEntry)(vcScan.elementAt(i));
			newTableData[i][0] = zEntry.getName() + (zEntry.isDirectory() ? "*" : "");
			int percCompression = (int)(Math.round(100.0f - (zEntry.getCompressedSize() / (zEntry.getSize() / 100.0f))));
			newTableData[i][1] = percCompression + "%";
			newTableData[i][2] = "(" + zEntry.getSize() + "/" + zEntry.getCompressedSize() + ")";
			newTableData[i][3] = (zEntry.getMethod() == ZipEntry.DEFLATED ? "[Y]" : (zEntry.getMethod() == ZipEntry.STORED ? "[N]" : "[?]"));
			calen.setTime(new Date(zEntry.getTime()));
			String dateString = calen.get(Calendar.MONTH) + "/" + calen.get(Calendar.DAY_OF_MONTH) + "/" + calen.get(Calendar.YEAR) + " " + calen.get(Calendar.HOUR) + ":" + calen.get(Calendar.MINUTE);
			newTableData[i][4] = dateString;
			newTableData[i][5] = zEntry.getExtra();
			newTableData[i][6] = zEntry.getComment();
			newTableData[i][7] = "<" + zEntry.getCrc() + ">";
		}
		zipTableModel.setTableData(newTableData);
		jtblArchiveInfo.revalidate();
	}

	private void saveArchive() throws FileNotFoundException, IOException
	{
		JFileChooser saveDialog = new JFileChooser(".");
		saveDialog.setDialogType(JFileChooser.SAVE_DIALOG);
		saveDialog.setFileFilter(new MutableFilter(extsZip, "Zip Archives (*.zip, *.gzip, *.gz)"));
		int optionSelected = saveDialog.showOpenDialog(this);
		if(optionSelected == JFileChooser.APPROVE_OPTION)
		{
			currentArchive = saveDialog.getSelectedFile();
			setAppTitle(currentArchive.getName());
			Vector vcFiles = new Vector();
			vcFiles.add(new File("alpha.txt"));
			vcFiles.add(new File("beta.txt"));
			vcFiles.add(new File("gamma.gif"));
			dstc.makeZip(currentArchive, vcFiles);
			openArchive(currentArchive);
			setAppTitle(currentArchive.getName());
		}
	}

	private void extractFileFromArchive() throws FileNotFoundException, IOException
	{
		if(jtblArchiveInfo.getSelectedRow() < 0)
		{
			sysMsg("NO FILE SELECTED");
		}
		else
		{
			jtblArchiveInfo.setRowSelectionInterval(jtblArchiveInfo.getSelectedRow(), jtblArchiveInfo.getSelectedRow());
			String filename = (String)(jtblArchiveInfo.getValueAt(jtblArchiveInfo.getSelectedRow(), zipTableModel.COLUMN_FILENAME));
			if(filename.endsWith("*"))
			{
				filename = filename.substring(0, filename.lastIndexOf("*"));
			}
			dstc.extractFile(currentArchive, new File(filename));
		}
	}

	private void extractAllFromArchive() throws FileNotFoundException, IOException
	{
		dstc.extractAllFiles(currentArchive);
	}

	private void addToArchive() throws FileNotFoundException, IOException
	{
		JFileChooser addDialog = new JFileChooser(".");
		addDialog.setDialogType(JFileChooser.OPEN_DIALOG);
		int optionSelected = addDialog.showOpenDialog(this);
		if(optionSelected == JFileChooser.APPROVE_OPTION)
		{
			File zipAddition = addDialog.getSelectedFile();
			dstc.addFile(currentArchive, zipAddition);
			openArchive(currentArchive);
			setAppTitle(currentArchive.getName());
		}
	}

	private void sysMsg(String msg)
	{
		System.out.println(msg);
	}

	private void setAppTitle(String archiveName)
	{
		this.setTitle(progName + (archiveName.length() > 0 ? " - " : "") + archiveName);
	}

//  MAIN METHOD ------------------------------------------------------------------>

	public static void main(String[] args)
	{
		DSTCshell dstc = new DSTCshell();
	}

//  INNER CLASSES ---------------------------------------------------------------->

	/* Class for providing JFileChooser with a FileFilter */
	class MutableFilter extends javax.swing.filechooser.FileFilter
	{

		private String[] acceptableExtensions;
		private String descriptor;

		public MutableFilter(String[] exts, String desc)
		{
			acceptableExtensions = exts;
			descriptor           = desc;
		}

		public boolean accept(File file)
		{
			if(file.isDirectory())
			{
				return true;
			}
			String fileName = file.getName();
			String fileExt = fileName.substring(fileName.lastIndexOf(".") + 1, fileName.length()).toLowerCase();
			if(fileExt != null)
			{
				for(int i = 0; i < acceptableExtensions.length; i++)
				{
					if(fileExt.equals(acceptableExtensions[i]))
					{
						return true;
					}
				}
				return false;
			}
			else
			{
				return false;
			}
		}
		public String getDescription()
		{
			return descriptor;
		}

	}

	/* Table model for ZIP archive information */
	class ZipInfoTableModel extends AbstractTableModel
	{

		private String[] columnNames = { "Name", "Perc", "(Orig/Comp)", "Method", "Date", "Extra", "Comm", "CRC" };
		private Object[][] data = { { "", "", "", "", "", "", "", "" } };

		public static final int COLUMN_FILENAME = 0;

		// Constructor

		public ZipInfoTableModel()
		{
		}

		// Custom Methods

		public void setTableData(Object[][] newData)
		{
			data = newData;
		}

		public void setTableColumns(String[] newCols)
		{
			columnNames = newCols;
		}

		// Extended Methods

		public int getColumnCount()
		{
			return columnNames.length;
		}

		public int getRowCount()
		{
			return data.length;
		}

		public String getColumnName(int col)
		{
			return columnNames[col];
		}

		public Object getValueAt(int row, int col)
		{
			return data[row][col];
		}

		public Class getColumnClass(int c)
		{
			if(getValueAt(0, c) == null)
			{
				return new Object().getClass();
			}
			return getValueAt(0, c).getClass();
		}

		public void setValueAt(Object value, int row, int col)
		{
			data[row][col] = value;
			fireTableCellUpdated(row, col);
		}

	}

}